@@ -22,71 +22,57 @@
 */
 
 #include <igraph.h>
-#include <math.h>
 
 int main(void) {
 
-    igraph_t g;
+    igraph_t graph;
     igraph_real_t cent;
-    igraph_arpack_options_t arpack_options;
 
-    /****************************/
-    /* undirected star */
-    igraph_star(&g, 10, IGRAPH_STAR_UNDIRECTED, /*center=*/ 0);
+    /* Create an undirected star graph, which is the most centralized graph
+     * with several common centrality scores. */
+    printf("undirected star graph:\n");
+    igraph_star(&graph, 10, IGRAPH_STAR_UNDIRECTED, /*center=*/ 0);
 
-    igraph_centralization_degree(&g, /*res=*/ NULL,
+    igraph_centralization_degree(&graph, /*res=*/ NULL,
                                  /*mode=*/ IGRAPH_ALL, IGRAPH_NO_LOOPS,
                                  &cent, /*theoretical_max=*/ NULL,
                                  /*normalized=*/ true);
-    if (cent != 1.0) {
-        fprintf(stderr, "undirected star, degree: %g\n", cent);
-        return 21;
-    }
+    printf("degree centralization: %g\n", cent);
 
-    igraph_centralization_betweenness(&g, /*res=*/ NULL,
+    igraph_centralization_betweenness(&graph, /*res=*/ NULL,
                                       IGRAPH_UNDIRECTED, &cent,
                                       /*theoretical_max=*/ NULL,
                                       /*normalized=*/ true);
-    if (cent != 1.0) {
-        fprintf(stderr, "undirected star, betweenness: %g\n", cent);
-        return 22;
-    }
+    printf("betweenness centralization: %g\n", cent);
 
-    igraph_centralization_closeness(&g, /*res=*/ NULL,
+
+    igraph_centralization_closeness(&graph, /*res=*/ NULL,
                                     IGRAPH_ALL, &cent,
                                     /*theoretical_max=*/ NULL,
                                     /*normalized=*/ true);
+    printf("closeness centralization: %g\n", cent);
 
-    if (!igraph_almost_equals(cent, 1.0, 1e-8)) {
-        fprintf(stderr, "undirected star, closeness: %g\n", cent);
-        return 23;
-    }
+    igraph_destroy(&graph);
 
-    igraph_destroy(&g);
+    /* With eigenvector centrality, the most centralized structure is
+     * a graph containing a single edge. */
+    printf("\ngraph with single edge:\n");
+    igraph_small(&graph, /*n=*/ 10, /*directed=*/ 0,
+                 0,1, -1);
 
-    /****************************/
-    /* single dyad */
-
-    igraph_small(&g, /*n=*/ 10, /*directed=*/ 0,
-                 0, 1, -1);
-
-    igraph_arpack_options_init(&arpack_options);
     igraph_centralization_eigenvector_centrality(
-                &g,
+                &graph,
                 /*vector=*/ NULL,
                 /*value=*/ NULL,
                 IGRAPH_DIRECTED,
                 /*scale=*/ true,
-                &arpack_options, &cent,
+                /*options=*/ NULL,
+                &cent,
                 /*theoretical_max=*/ NULL,
                 /*normalized=*/ true);
+    printf("eigenvector centralization: %g\n", cent);
 
-    if (!igraph_almost_equals(cent, 1.0, 1e-8)) {
-        fprintf(stderr, "dyad, eigenvector centrality: %g\n", cent);
-        return 24;
-    }
-
-    igraph_destroy(&g);
+    igraph_destroy(&graph);
 
     return 0;
 }
