@@ -210,7 +210,7 @@ igraph_error_t igraph_copy(igraph_t *to, const igraph_t *from) {
     IGRAPH_CHECK(igraph_i_property_cache_copy(to->cache, from->cache));
     IGRAPH_FINALLY(igraph_i_property_cache_destroy, to->cache);
 
-    IGRAPH_I_ATTRIBUTE_COPY(to, from, 1, 1, 1); /* does IGRAPH_CHECK */
+    IGRAPH_I_ATTRIBUTE_COPY(to, from, true, true, true); /* does IGRAPH_CHECK */
 
     IGRAPH_FINALLY_CLEAN(8);
     return IGRAPH_SUCCESS;
@@ -509,9 +509,9 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
 
     for (IGRAPH_EIT_RESET(eit); !IGRAPH_EIT_END(eit); IGRAPH_EIT_NEXT(eit)) {
         igraph_integer_t e = IGRAPH_EIT_GET(eit);
-        if (mark[e] == 0) {
+        if (! mark[e]) {
             edges_to_remove++;
-            mark[e]++;
+            mark[e] = true;
         }
     }
     remaining_edges = no_of_edges - edges_to_remove;
@@ -525,7 +525,7 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
 
     /* Actually remove the edges, move from pos i to pos j in newfrom/newto */
     for (i = 0, j = 0; j < remaining_edges; i++) {
-        if (mark[i] == 0) {
+        if (! mark[i]) {
             VECTOR(newfrom)[j] = VECTOR(graph->from)[i];
             VECTOR(newto)[j] = VECTOR(graph->to)[i];
             j++;
@@ -545,7 +545,7 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
         igraph_vector_int_t idx;
         IGRAPH_VECTOR_INT_INIT_FINALLY(&idx, remaining_edges);
         for (i = 0, j = 0; i < no_of_edges; i++) {
-            if (mark[i] == 0) {
+            if (! mark[i]) {
                 VECTOR(idx)[j++] = i;
             }
         }
@@ -622,7 +622,9 @@ igraph_error_t igraph_delete_edges(igraph_t *graph, igraph_es_t edges) {
  *     may contain the same ID more than once.
  * \param idx An optional pointer to a vector that provides the mapping from
  *     the vertex IDs \em before the removal to the vertex IDs \em after
- *     the removal. You can supply \c NULL here if you are not interested.
+ *     the removal, \em plus one. Zero is used to represent vertices that were
+ *     removed during the operation. You can supply \c NULL here if you are not
+ *     interested.
  * \param invidx An optional pointer to a vector that provides the mapping from
  *     the vertex IDs \em after the removal to the vertex IDs \em before
  *     the removal. You can supply \c NULL here if you are not interested.
@@ -883,9 +885,16 @@ igraph_error_t igraph_i_neighbors(const igraph_t *graph, igraph_vector_int_t *ne
         igraph_neimode_t mode, igraph_loops_t loops, igraph_multiple_t multiple) {
 #define DEDUPLICATE_IF_NEEDED(vertex, n)                                                 \
     if (should_filter_duplicates) {                                                        \
-        if ((loops == IGRAPH_NO_LOOPS && vertex == pnode) ||                               \
-                (loops == IGRAPH_LOOPS_ONCE && vertex == pnode && last_added == pnode) ||  \
-                (multiple == IGRAPH_NO_MULTIPLE && vertex == last_added)) {                \
+        if (                                                                               \
+            (loops == IGRAPH_NO_LOOPS && vertex == pnode) ||                               \
+            (loops == IGRAPH_LOOPS_ONCE && vertex == pnode && last_added == pnode)         \
+        ) {                                                                                \
+            length -= n;                                                                   \
+            if (loops == IGRAPH_LOOPS_ONCE) {                                              \
+                last_added = -1;                                                           \
+            }                                                                              \
+            continue;                                                                      \
+        } else if (multiple == IGRAPH_NO_MULTIPLE && vertex == last_added) {               \
             length -= n;                                                                   \
             continue;                                                                      \
         } else {                                                                           \
@@ -955,6 +964,7 @@ igraph_error_t igraph_i_neighbors(const igraph_t *graph, igraph_vector_int_t *ne
                 VECTOR(*neis)[idx++] = to;
             }
         }
+
         if (mode & IGRAPH_IN) {
             j = VECTOR(graph->is)[node + 1];
             for (i = VECTOR(graph->is)[node]; i < j; i++) {
@@ -998,6 +1008,9 @@ igraph_error_t igraph_i_neighbors(const igraph_t *graph, igraph_vector_int_t *ne
                 if (should_filter_duplicates && ((loops == IGRAPH_LOOPS_ONCE && n1 == pnode && last_added == pnode) ||
                         (multiple == IGRAPH_NO_MULTIPLE))) {
                     length--;
+                    if (loops == IGRAPH_LOOPS_ONCE) {
+                        last_added = -1;
+                    }
                     continue;
                 }
                 VECTOR(*neis)[idx++] = n2;
@@ -1191,7 +1204,8 @@ igraph_error_t igraph_degree_1(const igraph_t *graph, igraph_integer_t *deg,
  *
  * \sa \ref igraph_strength() for the version that takes into account
  * edge weights; \ref igraph_degree_1() to efficiently compute the
- * degree of a single vertex.
+ * degree of a single vertex; \ref igraph_maxdegree() if you only need
+ * the largest degree.
  *
  * \example examples/simple/igraph_degree.c
  */
@@ -1210,6 +1224,15 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
         IGRAPH_ERROR("Invalid mode for degree calculation.", IGRAPH_EINVMODE);
     }
 
+    if (! loops) {
+        /* If the graph is known not to have loops, we can use the faster
+         * loops == true code path, which has O(1) complexity instead of of O(d). */
+        if (igraph_i_property_cache_has(graph, IGRAPH_PROP_HAS_LOOP) &&
+            !igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_HAS_LOOP)) {
+            loops = true;
+        }
+    }
+
     nodes_to_calc = IGRAPH_VIT_SIZE(vit);
     if (!igraph_is_directed(graph)) {
         mode = IGRAPH_ALL;
@@ -1235,6 +1258,26 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
                 VECTOR(*res)[i] += (VECTOR(graph->is)[vid + 1] - VECTOR(graph->is)[vid]);
             }
         }
+    } else if (igraph_vs_is_all(&vids)) { /* no loops, calculating degree for all vertices */
+        // When calculating degree for all vertices, iterating over edges is faster
+        igraph_integer_t no_of_edges = igraph_ecount(graph);
+
+        if (mode & IGRAPH_OUT) {
+            for (igraph_integer_t edge = 0; edge < no_of_edges; ++edge) {
+                igraph_integer_t from = IGRAPH_FROM(graph, edge);
+                if (from != IGRAPH_TO(graph, edge)) {
+                    VECTOR(*res)[from]++;
+                }
+            }
+        }
+        if (mode & IGRAPH_IN) {
+            for (igraph_integer_t edge = 0; edge < no_of_edges; ++edge) {
+                igraph_integer_t to = IGRAPH_TO(graph, edge);
+                if (IGRAPH_FROM(graph, edge) != to) {
+                    VECTOR(*res)[to]++;
+                }
+            }
+        }
     } else { /* no loops */
         if (mode & IGRAPH_OUT) {
             for (IGRAPH_VIT_RESET(vit), i = 0;
@@ -1293,7 +1336,7 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
    in 'eid' if it is found; otherwise 'eid' is left intact.
    */
 
-#define BINSEARCH(start,end,value,iindex,edgelist,N,result,result_pos) \
+#define BINSEARCH(start, end, value, iindex, edgelist, N, result, result_pos) \
     do { \
         while ((start) < (end)) { \
             igraph_integer_t mid =(start)+((end)-(start))/2; \
@@ -1321,11 +1364,11 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
         igraph_integer_t start2 = VECTOR(graph->is)[xto]; \
         igraph_integer_t end2 = VECTOR(graph->is)[xto+1]; \
         igraph_integer_t N2 = end2; \
-        igraph_integer_t *nullptr = 0; \
+        igraph_integer_t *nullpointer = NULL; \
         if (end-start < end2-start2) { \
-            BINSEARCH(start, end, xto, graph->oi, graph->to, N, eid, nullptr); \
+            BINSEARCH(start, end, xto, graph->oi, graph->to, N, eid, nullpointer); \
         } else { \
-            BINSEARCH(start2, end2, xfrom, graph->ii, graph->from, N2, eid, nullptr); \
+            BINSEARCH(start2, end2, xfrom, graph->ii, graph->from, N2, eid, nullpointer); \
         } \
     } while (0)
 
@@ -1338,20 +1381,23 @@ igraph_error_t igraph_degree(const igraph_t *graph, igraph_vector_int_t *res,
 
 /**
  * \function igraph_get_eid
- * \brief Get the edge ID from the end points of an edge.
+ * \brief Get the edge ID from the endpoints of an edge.
  *
  * For undirected graphs \c from and \c to are exchangeable.
  *
  * \param graph The graph object.
  * \param eid Pointer to an integer, the edge ID will be stored here.
+ *        If \p error is false and no edge was found, <code>-1</code>
+ *        will be returned.
  * \param from The starting point of the edge.
  * \param to The end point of the edge.
- * \param directed Logical constant, whether to search for directed
+ * \param directed Boolean, whether to search for directed
  *        edges in a directed graph. Ignored for undirected graphs.
- * \param error Logical scalar, whether to report an error if the edge
- *        was not found. If it is false, then -1 will be assigned to \p eid.
- *        Note that invalid vertex IDs in input arguments (\p from or \p to)
- *        always return an error code.
+ * \param error Boolean, whether to report an error if the edge
+ *        was not found. If it is false, then <code>-1</code> will be
+ *        assigned to \p eid. Note that invalid vertex IDs in input
+ *        arguments (\p from or \p to) always trigger an error,
+ *        regardless of this setting.
  * \return Error code.
  * \sa \ref igraph_edge() for the opposite operation, \ref igraph_get_all_eids_between()
  *     to retrieve all edge IDs between a pair of vertices.
@@ -1430,9 +1476,9 @@ igraph_error_t igraph_get_eid(const igraph_t *graph, igraph_integer_t *eid,
  * \param eids Pointer to an initialized vector, the result is stored
  *        here. It will be resized as needed.
  * \param pairs Vector giving pairs of vertices to fetch the edges for.
- * \param directed Logical scalar, whether to consider edge directions
+ * \param directed Boolean, whether to consider edge directions
  *        in directed graphs. This is ignored for undirected graphs.
- * \param error Logical scalar, whether it is an error to supply
+ * \param error Boolean, whether it is an error to supply
  *        non-connected vertices. If false, then -1 is
  *        returned for non-connected pairs.
  * \return Error code.
@@ -1505,7 +1551,7 @@ igraph_error_t igraph_get_eids(const igraph_t *graph, igraph_vector_int_t *eids,
 #undef FIND_DIRECTED_EDGE
 #undef FIND_UNDIRECTED_EDGE
 
-#define FIND_ALL_DIRECTED_EDGES(graph,xfrom,xto,eidvec) \
+#define FIND_ALL_DIRECTED_EDGES(graph, xfrom, xto, eidvec) \
     do { \
         igraph_integer_t start = VECTOR(graph->os)[xfrom]; \
         igraph_integer_t end = VECTOR(graph->os)[xfrom+1]; \
@@ -1516,7 +1562,7 @@ igraph_error_t igraph_get_eids(const igraph_t *graph, igraph_vector_int_t *eids,
         igraph_integer_t eid = -1; \
         igraph_integer_t pos = -1; \
         if (end-start < end2-start2) { \
-            BINSEARCH(start, end, xto, graph->oi, graph->to, N, &eid,&pos); \
+            BINSEARCH(start, end, xto, graph->oi, graph->to, N, &eid, &pos); \
             while (pos >= 0 && pos < N) { \
                 eid = VECTOR(graph->oi)[pos++]; \
                 if (VECTOR(graph->to)[eid] != xto) { break; } \
@@ -1532,7 +1578,7 @@ igraph_error_t igraph_get_eids(const igraph_t *graph, igraph_vector_int_t *eids,
         } \
     } while (0)
 
-#define FIND_ALL_UNDIRECTED_EDGES(graph,from,to,eidvec) \
+#define FIND_ALL_UNDIRECTED_EDGES(graph, from, to, eidvec) \
     do { \
         igraph_integer_t xfrom1 = from > to ? from : to; \
         igraph_integer_t xto1 = from > to ? to : from; \
@@ -1551,7 +1597,7 @@ igraph_error_t igraph_get_eids(const igraph_t *graph, igraph_vector_int_t *eids,
  *        here. It will be resized as needed.
  * \param source The ID of the source vertex
  * \param target The ID of the target vertex
- * \param directed Logical scalar, whether to consider edge directions
+ * \param directed Boolean, whether to consider edge directions
  *        in directed graphs. This is ignored for undirected graphs.
  * \return Error code.
  *
@@ -1616,32 +1662,18 @@ igraph_error_t igraph_get_all_eids_between(
 igraph_error_t igraph_incident(const igraph_t *graph, igraph_vector_int_t *eids, igraph_integer_t pnode,
         igraph_neimode_t mode) {
     if (!igraph_is_directed(graph) || mode == IGRAPH_ALL) {
-        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_TWICE, IGRAPH_MULTIPLE);
+        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_TWICE);
     } else {
-        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_ONCE, IGRAPH_MULTIPLE);
+        return igraph_i_incident(graph, eids, pnode, mode, IGRAPH_LOOPS_ONCE);
     }
 }
 
 igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eids, igraph_integer_t pnode,
-        igraph_neimode_t mode, igraph_loops_t loops, igraph_multiple_t multiple) {
-#define DEDUPLICATE_IF_NEEDED(vertex, n)                                                 \
-    if (should_filter_duplicates) {                                                        \
-        if ((loops == IGRAPH_NO_LOOPS && vertex == pnode) ||                               \
-                (loops == IGRAPH_LOOPS_ONCE && vertex == pnode && last_added == pnode) ||  \
-                (multiple == IGRAPH_NO_MULTIPLE && vertex == last_added)) {                \
-            length -= n;                                                                   \
-            continue;                                                                      \
-        } else {                                                                           \
-            last_added = vertex;                                                           \
-        }                                                                                  \
-    }
+        igraph_neimode_t mode, igraph_loops_t loops) {
     igraph_integer_t length = 0, idx = 0;
     igraph_integer_t i, j;
-
     igraph_integer_t node = pnode;
-    igraph_integer_t last_added = -1;
-
-    igraph_bool_t should_filter_duplicates;
+    igraph_bool_t directed = igraph_is_directed(graph);
 
     if (node < 0 || node > igraph_vcount(graph) - 1) {
         IGRAPH_ERROR("Given vertex is not in the graph.", IGRAPH_EINVVID);
@@ -1651,7 +1683,7 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
         IGRAPH_ERROR("Mode should be either IGRAPH_OUT, IGRAPH_IN or IGRAPH_ALL.", IGRAPH_EINVMODE);
     }
 
-    if (!igraph_is_directed(graph)) {
+    if (!directed) {
         mode = IGRAPH_ALL;
     }
 
@@ -1679,29 +1711,32 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
      * an easy job. If we have requested both, we need to merge the two lists
      * to ensure that the output is sorted by the vertex IDs of the "other"
      * endpoint of the affected edges */
-    if (!igraph_is_directed(graph) || mode != IGRAPH_ALL) {
+    if (!directed || mode != IGRAPH_ALL) {
         /* We did not ask for both directions; this is the easy case */
 
-        should_filter_duplicates = !(multiple == IGRAPH_MULTIPLE &&
-                ((!igraph_is_directed(graph) && loops == IGRAPH_LOOPS_TWICE) ||
-                 (igraph_is_directed(graph) && loops != IGRAPH_NO_LOOPS)));
-
         if (mode & IGRAPH_OUT) {
             j = VECTOR(graph->os)[node + 1];
             for (i = VECTOR(graph->os)[node]; i < j; i++) {
                 igraph_integer_t edge = VECTOR(graph->oi)[i];
                 igraph_integer_t other = VECTOR(graph->to)[edge];
-                DEDUPLICATE_IF_NEEDED(other, 1);
-                VECTOR(*eids)[idx++] = edge;
+                if (loops == IGRAPH_NO_LOOPS && other == pnode) {
+                    length--;
+                } else {
+                    VECTOR(*eids)[idx++] = edge;
+                }
             }
         }
+
         if (mode & IGRAPH_IN) {
             j = VECTOR(graph->is)[node + 1];
             for (i = VECTOR(graph->is)[node]; i < j; i++) {
                 igraph_integer_t edge = VECTOR(graph->ii)[i];
                 igraph_integer_t other = VECTOR(graph->from)[edge];
-                DEDUPLICATE_IF_NEEDED(other, 1);
-                VECTOR(*eids)[idx++] = edge;
+                if ((loops == IGRAPH_NO_LOOPS || (loops == IGRAPH_LOOPS_ONCE && !directed)) && other == pnode) {
+                    length--;
+                } else {
+                    VECTOR(*eids)[idx++] = edge;
+                }
             }
         }
     } else {
@@ -1713,9 +1748,7 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
         igraph_integer_t i2 = VECTOR(graph->is)[node];
         igraph_integer_t eid1, eid2;
         igraph_integer_t n1, n2;
-
-        should_filter_duplicates = !(multiple == IGRAPH_MULTIPLE &&
-                loops == IGRAPH_LOOPS_TWICE);
+        igraph_bool_t seen_loop_edge = false;
 
         while (i1 < j1 && i2 < j2) {
             eid1 = VECTOR(graph->oi)[i1];
@@ -1724,37 +1757,44 @@ igraph_error_t igraph_i_incident(const igraph_t *graph, igraph_vector_int_t *eid
             n2 = VECTOR(graph->from)[eid2];
             if (n1 < n2) {
                 i1++;
-                DEDUPLICATE_IF_NEEDED(n1, 1);
                 VECTOR(*eids)[idx++] = eid1;
             } else if (n1 > n2) {
                 i2++;
-                DEDUPLICATE_IF_NEEDED(n2, 1);
                 VECTOR(*eids)[idx++] = eid2;
-            } else {
+            } else if (n1 != pnode) {
+                /* multiple edge */
                 i1++;
                 i2++;
-                DEDUPLICATE_IF_NEEDED(n2, 2);
                 VECTOR(*eids)[idx++] = eid1;
-                if (should_filter_duplicates && ((loops == IGRAPH_LOOPS_ONCE && n1 == pnode && last_added == pnode) ||
-                        (multiple == IGRAPH_NO_MULTIPLE))) {
-                    length--;
-                    continue;
-                }
                 VECTOR(*eids)[idx++] = eid2;
+            } else {
+                /* loop edge */
+                i1++;
+                i2++;
+                if (loops == IGRAPH_NO_LOOPS) {
+                    length -= 2;
+                } else if (loops == IGRAPH_LOOPS_ONCE) {
+                    length--;
+                    if (!seen_loop_edge) {
+                        VECTOR(*eids)[idx++] = eid1;
+                    } else {
+                        VECTOR(*eids)[idx++] = eid2;
+                    }
+                    seen_loop_edge = !seen_loop_edge;
+                } else {
+                    VECTOR(*eids)[idx++] = eid1;
+                    VECTOR(*eids)[idx++] = eid2;
+                }
             }
         }
 
         while (i1 < j1) {
             eid1 = VECTOR(graph->oi)[i1++];
-            igraph_integer_t to = VECTOR(graph->to)[eid1];
-            DEDUPLICATE_IF_NEEDED(to, 1);
             VECTOR(*eids)[idx++] = eid1;
         }
 
         while (i2 < j2) {
             eid2 = VECTOR(graph->ii)[i2++];
-            igraph_integer_t from = VECTOR(graph->from)[eid2];
-            DEDUPLICATE_IF_NEEDED(from, 1);
             VECTOR(*eids)[idx++] = eid2;
         }
     }
@@ -1803,7 +1843,7 @@ igraph_error_t igraph_is_same_graph(const igraph_t *graph1, const igraph_t *grap
     igraph_integer_t ne2 = igraph_ecount(graph2);
     igraph_integer_t i, eid1, eid2;
 
-    *res = 0; /* Assume that the graphs differ */
+    *res = false; /* Assume that the graphs differ */
 
     /* Check for same number of vertices/edges */
     if ((nv1 != nv2) || (ne1 != ne2)) {
@@ -1841,7 +1881,7 @@ igraph_error_t igraph_is_same_graph(const igraph_t *graph1, const igraph_t *grap
         }
     }
 
-    *res = 1; /* No difference was found, graphs are the same */
+    *res = true; /* No difference was found, graphs are the same */
     return IGRAPH_SUCCESS;
 }
 
