@@ -60,8 +60,8 @@
 typedef union YYSTYPE
 
 {
-  long int edgenum;
-  double weightnum;
+  igraph_integer_t edgenum;
+  igraph_real_t weightnum;
 }
 /* Line 1529 of yacc.c.  */
 
