@@ -24,6 +24,7 @@
 #include "igraph_structural.h"
 #include "igraph_topology.h"
 
+#include "igraph_bitset.h"
 #include "igraph_constructors.h"
 #include "igraph_dqueue.h"
 #include "igraph_interface.h"
@@ -65,13 +66,13 @@ igraph_error_t igraph_unfold_tree(const igraph_t *graph, igraph_t *tree,
     igraph_integer_t tree_vertex_count = no_of_nodes;
 
     igraph_vector_int_t edges;
-    igraph_vector_bool_t seen_vertices;
-    igraph_vector_bool_t seen_edges;
+    igraph_bitset_t seen_vertices;
+    igraph_bitset_t seen_edges;
 
     igraph_dqueue_int_t Q;
     igraph_vector_int_t neis;
 
-    igraph_integer_t i, n, r, v_ptr = no_of_nodes;
+    igraph_integer_t v_ptr = no_of_nodes;
 
     if (! igraph_vector_int_isininterval(roots, 0, no_of_nodes-1)) {
         IGRAPH_ERROR("All roots should be vertices of the graph.", IGRAPH_EINVVID);
@@ -81,44 +82,42 @@ igraph_error_t igraph_unfold_tree(const igraph_t *graph, igraph_t *tree,
     IGRAPH_CHECK(igraph_vector_int_reserve(&edges, no_of_edges * 2));
     IGRAPH_DQUEUE_INT_INIT_FINALLY(&Q, 100);
     IGRAPH_VECTOR_INT_INIT_FINALLY(&neis, 0);
-    IGRAPH_VECTOR_BOOL_INIT_FINALLY(&seen_vertices, no_of_nodes);
-    IGRAPH_VECTOR_BOOL_INIT_FINALLY(&seen_edges, no_of_edges);
+    IGRAPH_BITSET_INIT_FINALLY(&seen_vertices, no_of_nodes);
+    IGRAPH_BITSET_INIT_FINALLY(&seen_edges, no_of_edges);
 
     if (vertex_index) {
-        IGRAPH_CHECK(igraph_vector_int_resize(vertex_index, no_of_nodes));
-        for (i = 0; i < no_of_nodes; i++) {
-            VECTOR(*vertex_index)[i] = i;
-        }
+        IGRAPH_CHECK(igraph_vector_int_range(vertex_index, 0, no_of_nodes));
     }
 
-    for (r = 0; r < no_of_roots; r++) {
+    for (igraph_integer_t r = 0; r < no_of_roots; r++) {
 
         igraph_integer_t root = VECTOR(*roots)[r];
-        VECTOR(seen_vertices)[root] = 1;
+        IGRAPH_BIT_SET(seen_vertices, root);
         IGRAPH_CHECK(igraph_dqueue_int_push(&Q, root));
 
         while (!igraph_dqueue_int_empty(&Q)) {
             igraph_integer_t actnode = igraph_dqueue_int_pop(&Q);
 
             IGRAPH_CHECK(igraph_incident(graph, &neis, actnode, mode));
-            n = igraph_vector_int_size(&neis);
-            for (i = 0; i < n; i++) {
+
+            igraph_integer_t n = igraph_vector_int_size(&neis);
+            for (igraph_integer_t i = 0; i < n; i++) {
 
                 igraph_integer_t edge = VECTOR(neis)[i];
                 igraph_integer_t from = IGRAPH_FROM(graph, edge);
                 igraph_integer_t to = IGRAPH_TO(graph, edge);
                 igraph_integer_t nei = IGRAPH_OTHER(graph, edge, actnode);
 
-                if (! VECTOR(seen_edges)[edge]) {
+                if (! IGRAPH_BIT_TEST(seen_edges, edge)) {
 
-                    VECTOR(seen_edges)[edge] = 1;
+                    IGRAPH_BIT_SET(seen_edges, edge);
 
-                    if (! VECTOR(seen_vertices)[nei]) {
+                    if (! IGRAPH_BIT_TEST(seen_vertices, nei)) {
 
                         IGRAPH_CHECK(igraph_vector_int_push_back(&edges, from));
                         IGRAPH_CHECK(igraph_vector_int_push_back(&edges, to));
 
-                        VECTOR(seen_vertices)[nei] = 1;
+                        IGRAPH_BIT_SET(seen_vertices, nei);
                         IGRAPH_CHECK(igraph_dqueue_int_push(&Q, nei));
 
                     } else {
@@ -144,8 +143,8 @@ igraph_error_t igraph_unfold_tree(const igraph_t *graph, igraph_t *tree,
 
     } /* r < igraph_vector_int_size(roots) */
 
-    igraph_vector_bool_destroy(&seen_edges);
-    igraph_vector_bool_destroy(&seen_vertices);
+    igraph_bitset_destroy(&seen_edges);
+    igraph_bitset_destroy(&seen_vertices);
     igraph_vector_int_destroy(&neis);
     igraph_dqueue_int_destroy(&Q);
     IGRAPH_FINALLY_CLEAN(4);
@@ -162,13 +161,13 @@ igraph_error_t igraph_unfold_tree(const igraph_t *graph, igraph_t *tree,
 /* count the number of vertices reachable from the root */
 static igraph_error_t igraph_i_is_tree_visitor(const igraph_t *graph, igraph_integer_t root, igraph_neimode_t mode, igraph_integer_t *visited_count) {
     igraph_stack_int_t stack;
-    igraph_vector_bool_t visited;
+    igraph_bitset_t visited;
     igraph_vector_int_t neighbors;
     igraph_integer_t i;
 
     IGRAPH_VECTOR_INT_INIT_FINALLY(&neighbors, 0);
 
-    IGRAPH_VECTOR_BOOL_INIT_FINALLY(&visited, igraph_vcount(graph));
+    IGRAPH_BITSET_INIT_FINALLY(&visited, igraph_vcount(graph));
 
     IGRAPH_CHECK(igraph_stack_int_init(&stack, 0));
     IGRAPH_FINALLY(igraph_stack_int_destroy, &stack);
@@ -184,8 +183,8 @@ static igraph_error_t igraph_i_is_tree_visitor(const igraph_t *graph, igraph_int
 
         /* take a vertex from the stack, mark it as visited */
         u = igraph_stack_int_pop(&stack);
-        if (IGRAPH_LIKELY(! VECTOR(visited)[u])) {
-            VECTOR(visited)[u] = 1;
+        if (IGRAPH_LIKELY(! IGRAPH_BIT_TEST(visited, u))) {
+            IGRAPH_BIT_SET(visited, u);
             *visited_count += 1;
         }
 
@@ -194,7 +193,7 @@ static igraph_error_t igraph_i_is_tree_visitor(const igraph_t *graph, igraph_int
         ncount = igraph_vector_int_size(&neighbors);
         for (i = 0; i < ncount; ++i) {
             igraph_integer_t v = VECTOR(neighbors)[i];
-            if (! VECTOR(visited)[v]) {
+            if (! IGRAPH_BIT_TEST(visited, v)) {
                 IGRAPH_CHECK(igraph_stack_int_push(&stack, v));
             }
         }
@@ -202,7 +201,7 @@ static igraph_error_t igraph_i_is_tree_visitor(const igraph_t *graph, igraph_int
 
     igraph_vector_int_destroy(&neighbors);
     igraph_stack_int_destroy(&stack);
-    igraph_vector_bool_destroy(&visited);
+    igraph_bitset_destroy(&visited);
     IGRAPH_FINALLY_CLEAN(3);
 
     return IGRAPH_SUCCESS;
@@ -217,13 +216,14 @@ static igraph_error_t igraph_i_is_tree_visitor(const igraph_t *graph, igraph_int
  * An undirected graph is a tree if it is connected and has no cycles.
  *
  * </para><para>
- * In the directed case, a possible additional requirement is that all
- * edges are oriented away from a root (out-tree or arborescence) or all edges
+ * In the directed case, an additional requirement is that all edges
+ * are oriented away from a root (out-tree or arborescence) or all edges
  * are oriented towards a root (in-tree or anti-arborescence).
  * This test can be controlled using the \p mode parameter.
  *
  * </para><para>
- * By convention, the null graph (i.e. the graph with no vertices) is considered not to be a tree.
+ * By convention, the null graph (i.e. the graph with no vertices) is considered
+ * not to be connected, and therefore not a tree.
  *
  * \param graph The graph object to analyze.
  * \param res Pointer to a logical variable, the result will be stored
@@ -258,20 +258,29 @@ igraph_error_t igraph_is_tree(const igraph_t *graph, igraph_bool_t *res, igraph_
     vcount = igraph_vcount(graph);
     ecount = igraph_ecount(graph);
 
-    /* For undirected graphs and for directed graphs with mode == IGRAPH_ALL,
-     * we can return early if we know from the cache that the graph is weakly
-     * connected and is a forest. We can do this even if the user wants the
-     * root vertex because we always return zero as the root vertex for
-     * undirected graphs */
-    if (treat_as_undirected &&
-        igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_FOREST) &&
-        igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED) &&
-        igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_FOREST) &&
-        igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED)
-    ) {
-        is_tree = true;
-        iroot = 0;
-        goto success;
+    if (igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED)) {
+        igraph_bool_t weakly_connected = igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED);
+
+        if (weakly_connected) {
+            /* For undirected graphs and for directed graphs with mode == IGRAPH_ALL,
+             * we can return early if we know from the cache that the graph is weakly
+             * connected and is a forest. We can do this even if the user wants the
+             * root vertex because we always return zero as the root vertex for
+             * undirected graphs */
+            if (treat_as_undirected &&
+                igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_FOREST) &&
+                igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_FOREST)
+            ) {
+                is_tree = true;
+                iroot = 0;
+                goto success;
+            }
+        } else /* ! weakly_connected */ {
+            /* If the graph is not weakly connected, then it is neither an undirected
+             * not a directed tree. There is no root, so we can return early. */
+            is_tree = false;
+            goto success;
+        }
     }
 
     /* A tree must have precisely vcount-1 edges. */
@@ -321,7 +330,7 @@ igraph_error_t igraph_is_tree(const igraph_t *graph, igraph_bool_t *res, igraph_
         IGRAPH_CHECK(igraph_vector_int_init(&degree, 0));
         IGRAPH_FINALLY(igraph_vector_int_destroy, &degree);
 
-        IGRAPH_CHECK(igraph_degree(graph, &degree, igraph_vss_all(), mode == IGRAPH_IN ? IGRAPH_OUT : IGRAPH_IN, /* loops = */ 1));
+        IGRAPH_CHECK(igraph_degree(graph, &degree, igraph_vss_all(), mode == IGRAPH_IN ? IGRAPH_OUT : IGRAPH_IN, IGRAPH_LOOPS));
 
         for (i = 0; i < vcount; ++i) {
             if (VECTOR(degree)[i] == 0) {
@@ -353,7 +362,7 @@ igraph_error_t igraph_is_tree(const igraph_t *graph, igraph_bool_t *res, igraph_
 
     break;
     default:
-        IGRAPH_ERROR("Invalid mode,", IGRAPH_EINVMODE);
+        IGRAPH_ERROR("Invalid mode.", IGRAPH_EINVMODE);
     }
 
     /* if no suitable root was found, skip visiting vertices */
@@ -371,12 +380,12 @@ success:
         *root = iroot;
     }
 
-    if (is_tree && treat_as_undirected) {
-        /* For undirected graphs (or directed graphs that are treated as
-         * undirected in this calculation), a tree is weakly connected and is
-         * a forest, so we can cache this */
-        igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_FOREST, 1);
-        igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, 1);
+    if (is_tree) {
+        /* A graph that is a directed tree is also an undirected tree.
+         * An undirected tree is weakly connected and is a forest,
+         * so we can cache this. */
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_FOREST, true);
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_WEAKLY_CONNECTED, true);
     }
 
     return IGRAPH_SUCCESS;
@@ -392,7 +401,7 @@ success:
  */
 static igraph_error_t igraph_i_is_forest_visitor(
         const igraph_t *graph, igraph_integer_t root, igraph_neimode_t mode,
-        igraph_vector_bool_t *visited, igraph_stack_int_t *stack, igraph_vector_int_t *neis,
+        igraph_bitset_t *visited, igraph_stack_int_t *stack, igraph_vector_int_t *neis,
         igraph_integer_t *visited_count, igraph_bool_t *res)
 {
     igraph_integer_t i;
@@ -411,12 +420,12 @@ static igraph_error_t igraph_i_is_forest_visitor(
          * Otherwise mark it as visited and continue.
          */
         u = igraph_stack_int_pop(stack);
-        if (IGRAPH_LIKELY(! VECTOR(*visited)[u])) {
-            VECTOR(*visited)[u] = 1;
+        if (IGRAPH_LIKELY(! IGRAPH_BIT_TEST(*visited, u))) {
+            IGRAPH_BIT_SET(*visited, u);
             *visited_count += 1;
         }
         else {
-            *res = 0;
+            *res = false;
             break;
         }
 
@@ -438,12 +447,12 @@ static igraph_error_t igraph_i_is_forest_visitor(
                  * an already discovered vertex (i.e. one that has already been
                  * pushed onto the stack).
                  */
-                if (IGRAPH_LIKELY(! VECTOR(*visited)[v])) {
+                if (IGRAPH_LIKELY(! IGRAPH_BIT_TEST(*visited, v))) {
                     IGRAPH_CHECK(igraph_stack_int_push(stack, v));
                 }
                 /* To check for a self-loop in undirected graph */
                 else if (v == u) {
-                    *res = 0;
+                    *res = false;
                     break;
                 }
             }
@@ -466,22 +475,23 @@ static igraph_error_t igraph_i_is_forest(
  * \function igraph_is_forest
  * \brief Decides whether the graph is a forest.
  *
- * An undirected graph is a forest if it has no cycles.
- * </para><para>
+ * An undirected graph is a forest if it has no cycles. Equivalently,
+ * a graph is a forest if all connected components are trees.
  *
- * In the directed case, a possible additional requirement is that edges in each
+ * </para><para>
+ * In the directed case, an additional requirement is that edges in each
  * tree are oriented away from the root (out-trees or arborescences) or all edges
  * are oriented towards the root (in-trees or anti-arborescences).
  * This test can be controlled using the \p mode parameter.
- * </para><para>
  *
+ * </para><para>
  * By convention, the null graph (i.e. the graph with no vertices) is considered to be a forest.
- * </para><para>
  *
+ * </para><para>
  * The \p res return value of this function is cached in the graph itself if
  * \p mode is set to \c IGRAPH_ALL or if the graph is undirected. Calling the
  * function multiple times with no modifications to the graph in between
- * will return a cached value in O(1) time if the roots are not asked for.
+ * will return a cached value in O(1) time if the roots are not requested.
  *
  * \param graph The graph object to analyze.
  * \param res Pointer to a logical variable. If not \c NULL, then the result will be stored
@@ -504,28 +514,61 @@ static igraph_error_t igraph_i_is_forest(
  */
 igraph_error_t igraph_is_forest(const igraph_t *graph, igraph_bool_t *res,
                                 igraph_vector_int_t *roots, igraph_neimode_t mode) {
-    /* Caching is enabled only if the graph is undirected or mode == IGRAPH_ALL.
-     * Also, we can't return early if we need to calculate the roots */
-    igraph_bool_t use_cache = (
-        !igraph_is_directed(graph) || mode == IGRAPH_ALL
-    );
+
+    const igraph_bool_t treat_as_undirected = !igraph_is_directed(graph) || mode == IGRAPH_ALL;
 
     if (!roots && !res) {
         return IGRAPH_SUCCESS;
     }
 
-    if (use_cache && !roots && res) {
-        IGRAPH_RETURN_IF_CACHED_BOOL(graph, IGRAPH_PROP_IS_FOREST, res);
+    /* Note on cache use:
+     *
+     * The IGRAPH_PROP_IS_FOREST cached property is equivalent to this function's
+     * result ONLY in the undirected case. Keep in mind that a graph that is not
+     * a directed forest may still be an undirected forest, i.e. may still be free
+     * of undirected cycles. Example: 1->2<-3->4.
+     */
+
+    if (igraph_i_property_cache_has(graph, IGRAPH_PROP_IS_FOREST)) {
+        const igraph_bool_t no_undirected_cycles = igraph_i_property_cache_get_bool(graph, IGRAPH_PROP_IS_FOREST);
+        if (treat_as_undirected && res && ! roots) {
+            /* If the graph is treated as undirected and no roots are requested,
+             * we can directly use the cached IGRAPH_PROP_IS_FOREST value. */
+            *res = no_undirected_cycles;
+            return IGRAPH_SUCCESS;
+        } else {
+            /* Otherwise we can use negative cached values (i.e. "false"):
+             *  - A graph with undirected cycles cannot be a directed forest.
+             *  - If the graph is not a forest, we don't need to look for roots.
+             */
+            if (! no_undirected_cycles) {
+                if (res) { *res = false; }
+                if (roots) { igraph_vector_int_clear(roots); }
+                return IGRAPH_SUCCESS;
+            }
+        }
     }
 
     IGRAPH_CHECK(igraph_i_is_forest(graph, res, roots, mode));
 
-    /* At this point we know whether the graph is a forest if we have at least
-     * one of 'res' or 'roots' */
-    if (res) {
-        igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_FOREST, *res);
-    } else if (roots) {
-        igraph_i_property_cache_set_bool(graph, IGRAPH_PROP_IS_FOREST, !igraph_vector_int_empty(roots));
+    /* At this point we know whether the graph is an (undirected or directed) forest
+     * as we have at least one of 'res' or 'roots'. The case when both are NULL was
+     * caught above. */
+    igraph_bool_t is_forest;
+    if (res != NULL) {
+        is_forest = *res;
+    } else /* roots != NULL */ {
+        is_forest = igraph_vcount(graph) == 0 || !igraph_vector_int_empty(roots);
+    }
+    if (is_forest) {
+        /* If the graph is a directed forest, then it has no undirected cycles.
+         * We can enter positive results in the cache unconditionally. */
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_FOREST, true);
+    } else if (treat_as_undirected) {
+        /* However, if the graph is not a directed forest, it might still be
+         * an undirected forest. We can only enter negative results in the cache
+         * when edge directions were ignored, but NOT in the directed case. */
+        igraph_i_property_cache_set_bool_checked(graph, IGRAPH_PROP_IS_FOREST, false);
     }
 
     return IGRAPH_SUCCESS;
@@ -535,7 +578,7 @@ static igraph_error_t igraph_i_is_forest(
     const igraph_t *graph, igraph_bool_t *res,
     igraph_vector_int_t *roots, igraph_neimode_t mode
 ) {
-    igraph_vector_bool_t visited;
+    igraph_bitset_t visited;
     igraph_vector_int_t neis;
     igraph_stack_int_t stack;
     igraph_integer_t visited_count = 0;
@@ -578,7 +621,7 @@ static igraph_error_t igraph_i_is_forest(
 
     result = true; /* assume success */
 
-    IGRAPH_VECTOR_BOOL_INIT_FINALLY(&visited, vcount);
+    IGRAPH_BITSET_INIT_FINALLY(&visited, vcount);
 
     IGRAPH_CHECK(igraph_stack_int_init(&stack, 0));
     IGRAPH_FINALLY(igraph_stack_int_destroy, &stack);
@@ -605,7 +648,7 @@ static igraph_error_t igraph_i_is_forest(
                 if (!result) {
                     break;
                 }
-                if (! VECTOR(visited)[v]) {
+                if (! IGRAPH_BIT_TEST(visited, v)) {
                     if (roots) {
                         IGRAPH_CHECK(igraph_vector_int_push_back(roots, v));
                     }
@@ -670,7 +713,7 @@ static igraph_error_t igraph_i_is_forest(
 
     igraph_vector_int_destroy(&neis);
     igraph_stack_int_destroy(&stack);
-    igraph_vector_bool_destroy(&visited);
+    igraph_bitset_destroy(&visited);
     IGRAPH_FINALLY_CLEAN(3);
 
     return IGRAPH_SUCCESS;
@@ -688,6 +731,9 @@ static igraph_error_t igraph_i_is_forest(
         is stored here.
  * \return Error code.
  *
+ * \sa \ref igraph_find_cycle() to find a cycle that demonstrates
+ * that the graph is not acyclic.
+ *
  * Time complexity: O(|V|+|E|), where |V| and |E| are the number of
  * vertices and edges in the original input graph.
  */
