@@ -1,9 +1,6 @@
-/* -*- mode: C -*-  */
-/* vim:set ts=4 sw=4 sts=4 et: */
 /*
-   IGraph R library.
-   Copyright (C) 2003-2013  Gabor Csardi <csardi.gabor@gmail.com>
-   334 Harvard street, Cambridge, MA 02139 USA
+   IGraph library.
+   Copyright (C) 2003-2024  The igraph development team <igraph@igraph.org>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,10 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-   02110-1301 USA
-
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
 
 #include <igraph.h>
@@ -31,8 +25,8 @@ int main(void) {
 
     igraph_rng_seed(igraph_rng_default(), 9275);
 
-    igraph_erdos_renyi_game_gnp(&g1, 10, .3, IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
-    igraph_correlated_game(&g1, &g2, .9, .3, /* permutation=*/ 0);
+    igraph_erdos_renyi_game_gnp(&g1, 10, 0.3, IGRAPH_UNDIRECTED, IGRAPH_NO_LOOPS);
+    igraph_correlated_game(&g1, &g2, 0.9, 0.3, /* permutation=*/ NULL);
 
     IGRAPH_ASSERT(igraph_vcount(&g1) == igraph_vcount(&g2));
 
