@@ -17,14 +17,14 @@
 */
 
 #include <igraph.h>
-#include "test_utilities.inc"
+#include "test_utilities.h"
 
-int handler(const char* message, igraph_real_t percent, void*data) {
+igraph_error_t handler(const char* message, igraph_real_t percent, void*data) {
     printf("handler, %s, %f, %d\n", message, percent, *(int*)data);
     return IGRAPH_SUCCESS;
 }
 
-int main() {
+int main(void) {
     igraph_set_progress_handler(handler);
     int data = 10;
 
